#
# WrtWiki 
# Copyright 2005 Gerrit.Riessen@open-source-consultants.de
# Licensed under the GNU Public License V2.
#
BEGIN {
  # Arguments: (don't change order)
  ARGS[0] = "(lsCommand|ls)";
  ARGS_VALS[0] = "ls";
  ARGS_DESC[0] = "Command to use to perform a directory listing";
  ARGS[1] = "(fileDir|wikiFiles)";
  ARGS_VALS[1] = "wiki";
  ARGS_DESC[1] = "Directory containing the Wiki files (relative or absolute)";
  ARGS[2] = "(outputFile|output)";
  ARGS_VALS[2] = "stdout";
  ARGS_DESC[2] = "Output file.";
  ARGS[3] = "(ignoreFiles|dontBotherWith)";
  ARGS_VALS[3] = "(~$|^#.*#$|^__.*__$|httpd.conf)";
  ARGS_DESC[3] = "Ignore File names that match this regular expression";
  ARGS[4] = "(linkURL|URL)";
  ARGS_VALS[4] = "cgi-bin-wrt-wiki";
  ARGS_DESC[4] = "URL to use for doing the linking";
  
  # Gather the link words from the file directory.
  linkWords = " ";
  com = sprintf( "%s %s 2>&1", ARGS_VALS[0], ARGS_VALS[1] );
  while ( (com | getline line) > 0 )
  {
    if ( line !~ ARGS_VALS[3] )
      {
        linkWords = sprintf( " %s %s ", linkWords, line );
      }
  }
  close( com );

  # the wiki script name
  linkURL = ARGS_VALS[4];

  wUpper = "[A-Z][a-z_0-9]*[A-Z]+[a-zA-Z_0-9]*";
  wLower = "[a-z_0-9]+[A-Z][a-z_0-9]*[A-Z]+[a-zA-Z_0-9]*";

  listDepth = 0;
  lineIsList = 0;
  inPreFlag = 0;
}

function endList() {
  if ( ! lineIsList && listDepth > 0 ) {
    for ( i = 0; i < listDepth; i++ ) {
      printf("</ul>");
    }
    listDepth = 0;
  }
}

function strip( str ) {
  sub( "^[[:space:]]+", "", str );
  sub( "[[:space:]]+$", "", str );
  return str;
}

function removeHTML( str ) {
  gsub( "[<]", "\\&lt;", str );
  gsub( "[>]", "\\&gt;", str );
  return str;
}

# function sets the variables endToken and pipeLoc
function findPipeAndBracket( j, k ) {
  endToken = -1;
  pipeLoc = -1;
  for ( j = i; j <= NF; j++ ) {
    if ( match( $j, "[|]" ) ) {
      pipeLoc = j;
      for ( k = j; k <= NF; k++ ) {
        if ( match( $k, "]" )) {
          endToken = k;
          break;
        }
      }
      break;
    }
  }
  return (endToken > 0);
}

// {
  lineIsList = 0;
  postProcess = "";
}

## following rules handle entire lines

# escape line to be in a <pre> region
/^!/ {
  endList();
  if ( ! inPreFlag ) {
    printf "<pre>";
    inPreFlag = 1;
  }
  print removeHTML( substr( $0, 2 ) );
  next;
}

inPreFlag {
  print "</pre>";
  inPreFlag = 0;
}

# empty lines (or lines containing only space) is replaced with a <p>
/^[[:space:]]*$/ {
  # empty line
  endList();
  print "<p>";
  next;
}

# horizontal line
/^----$/ {
  endList();
  print "<hr noshade width=\"90%\">";
  next;
}

# HTML escape: any line beginning with a pipe is assumed to be HTML.
/^[|]/ {
  endList();
  print substr( $0, 2 );
  next;
}

# redirect pages
/^-[>][^[:space:]]*$/ {
  mwrd = substr( $0, 3 );
  if ( mwrd ~ "^http://" ) {
    printf( "This is a page redirect to <a href=\"%s\">%s</a>.", 
            mwrd, mwrd );
  } else {
    printf( "This is a page redirect to <a href=\"%s?pn=%s\">%s</a>.", 
            linkURL, mwrd, mwrd );
  }
  exit;
}

# headings
/^:/ {
  endList();
  match( $1, "[:]+" );
  printf( "<h%d>", RLENGTH );
  postProcess = postProcess "</h" RLENGTH ">";
  sub( "^[:]+[[:space:]]*", "" );
}

# list item
/^[*]/ {
  lineIsList = 1;
  match( $1, "[*]+" ); # RLENGTH == number of stars, hence the list depth

  if ( listDepth == 0 ) {
    for ( i = 0; i < RLENGTH; i++ ) { printf( "<ul>" );  }
  } else if ( listDepth < RLENGTH ) {
    for ( i = 0; i < RLENGTH - listDepth; i++ ) { printf( "<ul>" ); }
  } else { # listDepth > RLENGTH
    for ( i = 0; i < listDepth - RLENGTH; i++ ) { printf( "</ul>" ); }
  }
  listDepth = RLENGTH;

  postProcess = postProcess "</li>";
  printf "<li>";
  ## remove start stars from $0
  sub( "^[*]+[[:space:]]*", "" );
}

## fields are now parsed separately
{ 

  endList();
  $0 = removeHTML( $0 );

  for ( i = 1; i <= NF; i++ ) {
    
    post_token = "";

    if ( match ($i, "^!" ) ) {
      printf substr( $i, 2 ) post_token " ";
      continue;
    }

    if ( match( $i, "^[`]+" ) || match( $i,"^[']+") ) {
      if ( RLENGTH == 2 ) {
        printf ( "<i>" );
        $i = substr( $i, 3 );
      } else if ( RLENGTH == 3 ) {
        printf ( "<b>" );
        $i = substr( $i, 4 );
      } 
    } 

    if ( match( $i, "[']+$" ) || match( $i, "[`]+$" ) ) {
      if ( RLENGTH == 2 ) {
        post_token = post_token "</i>";
        $i = substr( $i, 0, length($i) - 2 );
      } else if ( RLENGTH == 3 ) {
        post_token = post_token "</b>";
        $i = substr( $i, 0, length($i) - 3 );
      }
    }

    if ( match( $i, "^!" ) ) {
      printf substr( $i, 2 ) post_token " ";
      continue;
    }

    if ( match( $i, "^[[].+[|].+]" ) ) {
      ## checking for [name|link]
      rest = substr( $i, RLENGTH + 1 );
      lnkName = substr( $i, 2, match($i, "[|]") - 2);
      lnkHref = substr( $i, match($i, "[|]" ) + 1 );
      sub( "].*$","", lnkHref);
      sub( "^pn=", sprintf( "%s?pn=", linkURL ), lnkHref );
      $i = sprintf( "<a href=\"%s\">%s</a>%s", lnkHref, lnkName, rest );
    } else if ( match( $i, "^[[][^]]+]" ) ) {
      ## checking for [makeThisALink]
      mwrd = substr( $i, RSTART+1, RLENGTH-2 );
      rest = substr( $i, RLENGTH+1 );
      if ( match( linkWords, sprintf( " %s ", mwrd ) ) ) {
        $i = sprintf( "<a href=\"%s?pn=%s\">%s</a>%s", linkURL,
                      mwrd,mwrd, rest);
      } else {
        $i = sprintf( "%s<a href=\"%s?pn=%s&ep=t\">?</a>%s",mwrd,linkURL,
                      mwrd, rest );
      }
    } else if ( match( $i, "^[[]" ) && findPipeAndBracket() ) {
      ## checking for the case [blah fubar | something://else ]
      ## prints immediately and modifies the value of i
      lnkName = substr( $i, 2 );
      for ( j = i + 1; j < pipeLoc; j++ ) {
        lnkName = lnkName " " $j;
      }
      lnkName = lnkName " " substr($pipeLoc, 0, match($pipeLoc, "[|]") - 1);
      
      lnkHref = substr( $pipeLoc, match( $pipeLoc, "[|]" ) + 1 );
      for ( j = pipeLoc + 1; j < endToken; j++ ) {
        lnkHref = lnkHref " " $j;
      }
      if ( pipeLoc != endToken ) {
        lnkHref = lnkHref " " substr($endToken, 0, match($endToken, "[]]")-1);
      }

      lnkHref = strip( lnkHref );
      sub( "].*$","", lnkHref);
      sub( "^pn=", sprintf( "%s?pn=", linkURL ), lnkHref );
      
      rest = substr( $endToken, match( $endToken, "[]]" ) + 1);
      printf( "<a href=\"%s\">%s</a>%s%s ", lnkHref, strip(lnkName), 
              rest, post_token );
      i = endToken;
      continue;

    } else if ( match( $i, wUpper ) || match( $i, wLower ) ) {
      mwrd = substr( $i, RSTART, RLENGTH );
      mstart = RSTART;
      mlength = RLENGTH;
      if ( match( linkWords, sprintf( " %s ", mwrd ) ) ) {
        $i = sprintf( "%s<a href=\"%s?pn=%s\">%s</a>%s", substr($i,0,mstart-1),
                      linkURL, mwrd, mwrd, substr($i,mlength+mstart));
      } else {
        $i = sprintf( "%s%s<a href=\"%s?pn=%s&ep=t\">?</a>%s", 
                      substr($i,0,mstart-1), mwrd, linkURL, mwrd, 
                      substr( $i, mlength+mstart));
      }
    } else if (match($i, "(img://|https?://|ftp://|mailto:)([[:alnum:]]|[/_.%$@#~]|[-])+")) {
      mwrd = substr( $i, RSTART, RLENGTH );
      if ( substr(mwrd,0,6) == "img://" ) {
        mwrd = substr( mwrd, 7 );
        $i = sprintf( "%s<img src=\"%s\">%s", substr($i,0,RSTART-1), 
                      mwrd, substr($i, RSTART+RLENGTH ) );
      } else {
        $i = sprintf( "%s<a href=\"%s\">%s</a>%s", substr($i,0,RSTART-1), mwrd,
                      mwrd, substr($i,RSTART+RLENGTH ) );
      }
    }
    
    printf $i post_token " ";
  }

  print postProcess;
}


END {
  for ( i = 0; i < listDepth; i++ ) {
    printf("</ul>");
  }
  if ( inPreFlag ) print "</pre>";
}
